unit AlarmTicket;

interface

uses
  SysUtils, Classes, Forms, Controls, StdCtrls, Buttons, IniFiles,
  Ticket;

type
	TAlarmTime = record
  	Value: TDateTime;
    Elapsed: Boolean;
  end;
  TFormAlarmTicket = class(TFormTicket)
    BtnStop: TSpeedButton;
    LabelAlarmTime: TLabel;
    procedure BtnStopClick(Sender: TObject);
    procedure LabelAlarmTimeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
  	AlarmTime: TAlarmTime;
  protected
    procedure HandleTick; override;
    procedure SaveImage(TicketImage: TStringList); override;
  public
		function QueryAlarmTime: Boolean;
    procedure Save(       IniFile: TIniFile;
    							 const	Section: string); override;
    procedure Restore(      IniFile: TIniFile;
    							 		const	Section: string); override;
    procedure Assign(Source: TPersistent); override;
  end;

implementation

{$R *.DFM}

uses
	Windows, Graphics, Dialogs,
  Util, Main;

const
	TimeSpace = '     ';
  TimeSaveFormat = '%4.4d/%2.2d/%2.2d %2.2d:%2.2d';

procedure TFormAlarmTicket.HandleTick;
begin

	if AlarmTime.Value = 0 then
    TickEnabled := false
  else if AlarmTime.Elapsed then begin
	  LabelAlarmTime.Color := clRed;
    Show;
    MessageBeep(MB_ICONEXCLAMATION);
  end
	else begin
		if Now >= AlarmTime.Value then begin
 	    AlarmTime.Elapsed := true;
		  LabelAlarmTime.Color := clRed;
      Show;
     	MessageBeep(MB_ICONEXCLAMATION);
		end;
 end;

end;

function TFormAlarmTicket.QueryAlarmTime: Boolean;

var
	S: string;
  Done: Boolean;

begin

  Done := false;
  Result := false;
  repeat
  	DateTimeToString(S,ShortDateFormat + ' HH:mm',Now);
    if InputQuery('Alarm Time','',S) then begin
    	try
      	AlarmTime.Value := StrToDateTime(S);
  			AlarmTime.Elapsed := false;
       	DateTimeToString(S,ShortDateFormat +  ' HH:mm',AlarmTime.Value);
   		  LabelAlarmTime.Caption := TimeSpace + S;
  		  LabelAlarmTime.Color := clYellow;
        HandleTick;
        TickEnabled := (AlarmTime.Value <> 0);
        Done := true;
        Result := true;
      except
      	Done := (MessageDlg('Invalid Time',mtWarning,[mbCancel,mbRetry],0) = mrCancel);
      end;
    end
    else
    	Done := true;
  until Done;

end;

procedure TFormAlarmTicket.Save(				IniFile: TIniFile;
    							 							const 	Section: string);

var
	S: string;
  Year,Month,Day,Hour,Min,Sec,mSec: Word;

begin

	inherited Save(IniFile,Section);
	with IniFile do begin
    DecodeDate(AlarmTime.Value,Year,Month,Day);
    DecodeTime(AlarmTime.Value,Hour,Min,Sec,mSec);
  	S := Format(TimeSaveFormat,[Year,Month,Day,Hour,Min]);
    if AlarmTime.Elapsed then
    	S := S + ' 1'
    else
    	S := S + ' 0';
  	WriteString(Section,'AlarmTime',S);
  end;

end;

procedure TFormAlarmTicket.Restore(				IniFile: TIniFile;
 							 										 const	Section: string);

var
	S: string;
  Year,Month,Day,Hour,Min,Sec,mSec: Word;

begin

	inherited Restore(IniFile,Section);
	with IniFile do begin
	  S := ReadString(Section,'AlarmTime','');
    if S = '' then begin
			Year := 0;
      AlarmTime.Elapsed := true;
    end
    else begin
	    try
				Year := StrToInt(ParseStr(S,'/'));
				Month := StrToInt(ParseStr(S,'/'));
				Day := StrToInt(ParseStr(S,' '));
				Hour := StrToInt(ParseStr(S,':'));
				Min := StrToInt(ParseStr(S,' '));
        AlarmTime.Elapsed := (S[1] = '1');
	    except
				Year := 0;
	      AlarmTime.Elapsed := true;
	    end;
    end;
    if Year = 0 then begin
    	AlarmTime.Value := 0;
		  LabelAlarmTime.Caption := '';
      TickEnabled := false;
		end
    else begin
	    AlarmTime.Value := EncodeDate(Year,Month,Day) + EncodeTime(Hour,Min,0,0);
     	DateTimeToString(S,ShortDateFormat +  ' HH:mm',AlarmTime.Value);
 		  LabelAlarmTime.Caption := TimeSpace + S;
      TickEnabled := true;
    end;
	end;

end;

procedure TFormAlarmTicket.Assign(Source: TPersistent);
begin

  inherited Assign(Source);
	with Source as TFormAlarmTicket do begin
    Self.AlarmTime := AlarmTime;
    Self.LabelAlarmTime.Caption := LabelAlarmTime.Caption;
  end;

end;

procedure TFormAlarmTicket.SaveImage(TicketImage: TStringList);
begin

  inherited SaveImage(TicketImage);
	with TicketImage do
    Insert(1,'Alarm at: ' +  FormatDateTime(ShortDateFormat +  ' HH:mm',AlarmTime.Value));

end;

procedure TFormAlarmTicket.BtnStopClick(Sender: TObject);
begin

  TickEnabled := false;

end;

procedure TFormAlarmTicket.LabelAlarmTimeClick(Sender: TObject);
begin

	QueryAlarmTime;

end;

procedure TFormAlarmTicket.FormCreate(Sender: TObject);
begin

  inherited;
  LabelAlarmTime.Color := clYellow;

end;

initialization

	TFormAlarmTicket.Register;

end.

