unit Main;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Forms, Controls,
  StdCtrls, Buttons, IniFiles, ThdTimer, TrayIcon, Menus,
  ResIcon, Ticket, Dialogs;

type
  TTicketClass = class of TFormTicket;
  TFormMain = class(TForm)
    BtnNew: TSpeedButton;
    BtnTile: TSpeedButton;
    BtnNewTimed: TSpeedButton;
    BtnNewAlarm: TSpeedButton;
    TimerTick: TThreadedTimer;
    ComboBoxTickets: TComboBox;
    BtnShow: TSpeedButton;
    BtnCopy: TSpeedButton;
    TrayIcon: TTrayIcon;
    TrayMenu: TPopupMenu;
    ItemShow: TMenuItem;
    ItemExit: TMenuItem;
    N1: TMenuItem;
    ItemSave: TMenuItem;
    ItemReorder: TMenuItem;
    N2: TMenuItem;
    ItemPrintAll: TMenuItem;
    DialogSave: TSaveDialog;
    procedure BtnNewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnNewTimedMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnNewAlarmMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnCopyMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnShowClick(Sender: TObject);
    procedure BtnTileClick(Sender: TObject);
    procedure ComboBoxTicketsClick(Sender: TObject);
    procedure ComboBoxTicketsChange(Sender: TObject);
    procedure TimerTickTimer(Sender: TObject);
    procedure ItemShowClick(Sender: TObject);
    procedure ItemReorderClick(Sender: TObject);
    procedure ItemPrintAllClick(Sender: TObject);
    procedure ItemSaveClick(Sender: TObject);
    procedure ItemExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
  	FResourceIcon: TResourceIcon;
    TicketsFile: TIniFile;
    TopDelta,LeftDelta: Integer;
    LastActiveTicketIndex: Integer;
    procedure BtnEnabled(En,CopyEn: Boolean);
		procedure TicketNewPosition(var	NewTop,NewLeft: Integer);
		function TicketCreate(TicketClass: TTicketClass): TFormTicket;
    procedure TicketClose(		Sender: TObject;
    											var Action: TCloseAction);
		procedure TicketDeactivate(Sender: TObject);
    procedure TicketCount(Sender: TObject);
    procedure WMSysCommand(var Message: TMessage); message WM_SYSCOMMAND;
 		procedure TopMostExceptionHandler(Sender: TObject;
																			E: Exception);
  public
   	property ResourceIcon: TResourceIcon read FResourceIcon;
    procedure Print(TicketImage: TStringList;
                    Font: TFont);
    procedure SaveToFile(      TicketImage: TStringList;
                         const TicketCaption: string);
    procedure SetTray(IconID: string;
                      Hint: string);
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

uses
  FileCtrl, Printers,
  TimedTicket, AlarmTicket;

type
	EParam = class(Exception);

procedure TFormMain.BtnEnabled(En,CopyEn: Boolean);
begin

  BtnNew.Enabled := En;
  BtnNewAlarm.Enabled := En;
  BtnNewTimed.Enabled := En;
  BtnCopy.Enabled := CopyEn;

end;

procedure TFormMain.TicketNewPosition(var	NewTop,NewLeft: Integer);

var
	I: Integer;

begin

 	NewTop := Self.Top + Self.Height;
  NewLeft := Self.Left;
  I := 0;
  while I < ComboBoxTickets.Items.Count do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do begin
  	if Tag >= 0 then begin
	  	if (Abs(Top - NewTop) < TopDelta) or (Abs(Left - NewLeft) < LeftDelta) then
      	I := 0
      else
      	Inc(I);
	  	if Abs(Top - NewTop) < TopDelta then
  	  	NewTop := Top + TopDelta;
  		if Abs(Left - NewLeft) < LeftDelta then
    		NewLeft := Left + LeftDelta;
    end
    else
    	Inc(I);
  end;
  NewLeft := NewLeft;

end;

function TFormMain.TicketCreate(TicketClass: TTicketClass): TFormTicket;

var

	NewTop,NewLeft: Integer;

begin

  Result := TicketClass.Create(Self);
	TicketNewPosition(NewTop,NewLeft);
	with Result do begin
	  ListIndex := 0;
  	Caption := ComboBoxTickets.Text;
    ComboBoxTickets.Text := '';
  	Top := NewTop;
  	Left := NewLeft;
    Width := Self.Width;
  	OnClose := TicketClose;
    OnDeactivate := TicketDeactivate;
  	Show;
  end;
  if ComboBoxTickets.Items.Count = 0 then
		SetTray('TICKETICON',Application.Title);
	Result.ListIndex := ComboBoxTickets.Items.AddObject(Result.Caption,Result);

end;

procedure TFormMain.TicketClose(		Sender: TObject;
															  var Action: TCloseAction);

var
	I: Integer;

begin

  with Sender as TFormTicket do begin
  	ComboBoxTickets.Items.Delete(ListIndex);
    for I := ListIndex to ComboBoxTickets.Items.Count - 1 do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do
    	ListIndex := ListIndex - 1;
  end;
  if ComboBoxTickets.Items.Count = 0 then
		SetTray('MAINICON',Application.Title);
  Action := caFree;

end;

procedure TFormMain.TicketDeactivate(Sender: TObject);
begin

	LastActiveTicketIndex := TFormTicket(Sender).ListIndex;
  BtnEnabled((ComboBoxTickets.Text <> ''),
             ((ComboBoxTickets.Text <> '') and
   						(LastActiveTicketIndex >= 0) and
              (LastActiveTicketIndex < ComboBoxTickets.Items.Count)));

end;

procedure TFormMain.TicketCount(Sender: TObject);

var
	I: Integer;

begin

  for I := 0 to ComboBoxTickets.Items.Count - 1 do
  	if ComboBoxTickets.Items.Objects[I] is TFormTimedTicket then
			if ComboBoxTickets.Items.Objects[I] <> Sender then
        TFormTimedTicket(ComboBoxTickets.Items.Objects[I]).CountEn := false;

end;

procedure TFormMain.TimerTickTimer(Sender: TObject);

var
	I: Integer;

begin

  for I := 0 to ComboBoxTickets.Items.Count - 1 do
   	TFormTicket(ComboBoxTickets.Items.Objects[I]).Tick;

end;

procedure TFormMain.WMSysCommand(var Message: TMessage);

var
  I: Integer;

begin

	if Message.WParam = SC_MINIMIZE then begin
    if BtnShow.Down then begin
      BtnShow.Down := false;
      BtnShowClick(BtnShow);
    end;
  	for I := 0 to ComboBoxTickets.Items.Count - 1 do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do
      if Tag >= 0 then
    	  Hide;
    Hide;
    ItemShow.Enabled := true;
  end
  else
  	inherited;

end;

procedure TFormMain.SaveToFile(      TicketImage: TStringList;
                               const TicketCaption: string);
begin

  DialogSave.FileName := TicketCaption + '.txt';
  if DialogSave.Execute then try
    TicketImage.SaveToFile(DialogSave.FileName);
  except
  end;

end;

procedure TFormMain.Print(TicketImage: TStringList;
                          Font: TFont);

const
	LeftMargin = 200;
  TopMargin = 300;
  BottomMargin = 200;

var
	Y,DY,I: Integer;

begin

	with TicketImage, Printer do begin
  	Canvas.Font := Font;
  	DY := Canvas.TextHeight('I');
    Y := TopMargin;
	  BeginDoc;
    for I := 0 to Count - 1 do begin
      if Y > (PageHeight - BottomMargin) then begin
      	NewPage;
        Y := TopMargin;
      end;
			Canvas.TextOut(LeftMargin, Y, Strings[I]);
      Y := Y + DY;
    end;
		EndDoc;
  end;

end;

procedure TFormMain.SetTray(IconID: string;
                            Hint: string);
begin

  TrayIcon.Icon := ResourceIcon[IconID];
  TrayIcon.ToolTip := Hint;

end;

procedure TFormMain.BtnNewMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin

	TicketCreate(TFormTicket);
  BtnEnabled(false,false);

end;

procedure TFormMain.BtnNewTimedMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

	TFormTimedTicket(TicketCreate(TFormTimedTicket)).OnCountEn := TicketCount;
  BtnEnabled(false,false);

end;

procedure TFormMain.BtnNewAlarmMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

var
	New: TFormAlarmTicket;
	Action: TCloseAction;

begin

	New := TFormAlarmTicket(TicketCreate(TFormAlarmTicket));
  if not New.QueryAlarmTime then begin
		TicketClose(New,Action);
    New.Free;
  end;
  BtnEnabled(false,false);

end;

procedure TFormMain.BtnCopyMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);

type
	TTicketClass = class of TFormTicket;

var
	New: TFormTicket;

begin

	if (LastActiveTicketIndex >= 0) and (LastActiveTicketIndex < ComboBoxTickets.Items.Count) then begin
		New := TicketCreate(TTicketClass(ComboBoxTickets.Items.Objects[LastActiveTicketIndex].ClassType));
    New.Assign(TFormTicket(ComboBoxTickets.Items.Objects[LastActiveTicketIndex]));
    BtnEnabled(false,false);
  end;

end;

procedure TFormMain.BtnShowClick(Sender: TObject);

var
	I: Integer;

begin

  if BtnShow.Down then begin
    BtnEnabled(false,false);
		for I := 0 to ComboBoxTickets.Items.Count - 1 do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do
	  	if Tag < 0 then begin
		  	Show;
    	  Tag := -1;
    	end;
  end
	else begin
		for I := 0 to ComboBoxTickets.Items.Count - 1 do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do
	  	if Tag < 0 then
      	Hide;
    BtnEnabled((ComboBoxTickets.Text <> ''),
               ((ComboBoxTickets.Text <> '') and
    						(LastActiveTicketIndex >= 0) and
                (LastActiveTicketIndex < ComboBoxTickets.Items.Count)));
  end;

end;

procedure TFormMain.BtnTileClick(Sender: TObject);

var
	I,SlideTop,SlideLeft: Integer;

begin

	SlideTop := Self.Top + Self.Height;
  SlideLeft := Self.Left;
	for I := 0 to ComboBoxTickets.Items.Count - 1 do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do begin
  	if (Tag >= 0) or ((Tag < 0) and BtnShow.Down) then begin
    	Top := SlideTop;
    	Left := SlideLeft;
  		SlideTop := SlideTop + TopDelta;
    	SlideLeft := SlideLeft + LeftDelta;
      Show;
    end;
  end;

end;

procedure TFormMain.ComboBoxTicketsClick(Sender: TObject);
begin

	if (ComboBoxTickets.ItemIndex >= 0) and (ComboBoxTickets.ItemIndex < ComboBoxTickets.Items.Count) then begin
		TFormTicket(ComboBoxTickets.Items.Objects[ComboBoxTickets.ItemIndex]).Show;
  	ComboBoxTickets.ItemIndex := -1;
  end;

end;

procedure TFormMain.ComboBoxTicketsChange(Sender: TObject);
begin

	if BtnShow.Down then
  	ComboBoxTickets.Text := '';
	if ComboBoxTickets.Text <> '' then
    BtnEnabled(true,((LastActiveTicketIndex >= 0) and (LastActiveTicketIndex < ComboBoxTickets.Items.Count)))
  else
    BtnEnabled(false,false);

end;

procedure TFormMain.ItemShowClick(Sender: TObject);

var
	I: Integer;

begin

  ItemShow.Enabled := false;
  Show;
  SetForegroundWindow(Handle);
	for I := 0 to ComboBoxTickets.Items.Count - 1 do with TFormTicket(ComboBoxTickets.Items.Objects[I]) do
  	if Tag >= 0 then
	  	Show;

end;

procedure TFormMain.ItemReorderClick(Sender: TObject);

var
	I: Integer;
  SortList: TStringList;

begin

  Enabled := false;
  TimerTick.Enabled := false;
  Application.ProcessMessages;
	SortList := TStringList.Create;
  SortList.Assign(ComboBoxTickets.Items);
  ComboBoxTickets.Clear;
  SortList.Sort;
  for I := 0 to SortList.Count - 1 do
  	TFormTicket(SortList.Objects[I]).ListIndex := I;
  ComboBoxTickets.Items.Assign(SortList);
  SortList.Free;
  TimerTick.Enabled := true;
  Enabled := true;

end;

procedure TFormMain.ItemPrintAllClick(Sender: TObject);

var
	I: Integer;

begin

  for I := 0 to ComboBoxTickets.Items.Count - 1 do
   	TFormTicket(ComboBoxTickets.Items.Objects[I]).Print;

end;

procedure TFormMain.ItemSaveClick(Sender: TObject);

var
	I,N_1: Integer;

begin

  Enabled := false;
  TimerTick.Enabled := false;
  Application.ProcessMessages;
	if TicketsFile <> nil then with TicketsFile do begin
    N_1 := ReadInteger('Tickets','N',0);
	  WriteInteger('Self','Top',Self.Top);
  	WriteInteger('Self','Left',Self.Left);
  	WriteInteger('Tickets','N',ComboBoxTickets.Items.Count);
    for I := 0 to ComboBoxTickets.Items.Count - 1 do
    	TFormTicket(ComboBoxTickets.Items.Objects[I]).Save(TicketsFile,Format('%2.2d',[I]));
    for I := ComboBoxTickets.Items.Count to N_1 - 1 do
      EraseSection(Format('%2.2d',[I]));
	end;
  TimerTick.Enabled := true;
  Enabled := true;

end;

procedure TFormMain.ItemExitClick(Sender: TObject);
begin

  Close;

end;

procedure TFormMain.TopMostExceptionHandler(Sender: TObject;
																						E: Exception);
begin

	Application.ShowException(E);
	Application.Terminate;

end;

procedure TFormMain.FormCreate(Sender: TObject);

var
	I,J: Integer;
	NewTicket: TFormTicket;
  TicketCreateInfo: TFormTicketCreateInfo;

begin

  Application.OnException := TopMostExceptionHandler;
  PostMessage(Handle,WM_SYSCOMMAND,SC_MINIMIZE,0);
	TopDelta := Self.Height - Self.ClientHeight;
  LeftDelta := TopDelta;
  FResourceIcon := TResourceIcon.Create;
  if not DirectoryExists(ExtractFilePath(ParamStr(1))) then
  	raise EParam.Create('Command line must specify the data file')
  else begin
    TicketsFile := TIniFile.Create(ParamStr(1));
  	Self.Top := TicketsFile.ReadInteger('Self','Top',0);
  	Self.Left := TicketsFile.ReadInteger('Self','Left',0);
    TicketCreateInfo := TFormTicketCreateInfo.Create;
    TicketCreateInfo.IniFile := TicketsFile;
	  for I := 0 to TicketsFile.ReadInteger('Tickets','N',0) - 1 do begin
      TicketCreateInfo.Section := Format('%2.2d',[I]);
      NewTicket := TFormTicket.DerivedClass(TicketCreateInfo).Create(Self);
      if NewTicket is TFormTimedTicket then
        TFormTimedTicket(NewTicket).OnCountEn := TicketCount;
    	with NewTicket do begin
	    	ListIndex := I;
        Restore(TicketsFile,Format('%2.2d',[I]));
		  	OnClose := TicketClose;
        OnDeactivate := TicketDeactivate;
	    end;
			ComboBoxTickets.Items.AddObject(NewTicket.Caption,NewTicket);
	  end;
    TicketCreateInfo.Free;
  	if ComboBoxTickets.Items.Count = 0 then
	 		SetTray('MAINICON',Application.Title)
	  else
	 		SetTray('TICKETICON',Application.Title);
  end;
  LastActiveTicketIndex := -1;
  BtnEnabled(false,false);
  TimerTick.Enabled := true;

end;

procedure TFormMain.FormDestroy(Sender: TObject);

var
	I,N_1: Integer;

begin

  Enabled := false;
  TimerTick.Enabled := false;
  Application.ProcessMessages;
	if TicketsFile <> nil then with TicketsFile do begin
    N_1 := ReadInteger('Tickets','N',0);
	  WriteInteger('Self','Top',Self.Top);
  	WriteInteger('Self','Left',Self.Left);
  	WriteInteger('Tickets','N',ComboBoxTickets.Items.Count);
    for I := 0 to ComboBoxTickets.Items.Count - 1 do
    	TFormTicket(ComboBoxTickets.Items.Objects[I]).Save(TicketsFile,Format('%2.2d',[I]));
		for I := 0 to ComboBoxTickets.Items.Count - 1 do
  		ComboBoxTickets.Items.Objects[I].Free;
    for I := ComboBoxTickets.Items.Count to N_1 - 1 do
      EraseSection(Format('%2.2d',[I]));
	  Free;
	end;
  FResourceIcon.Free;

end;


end.

