unit Ticket;

interface

uses
  SysUtils, Classes, Windows, Messages, Forms, Controls, StdCtrls, IniFiles, ComCtrls, Menus,
  Deriver;

type
 	TFormTicketCreateInfo = class(TObject)
  public
		IniFile: TIniFile;
    Section: string;
  end;
  TFormTicket = class;
  TFormTicketClass = class of TFormTicket;
  TFormTicket = class(TForm)
    Memo: TRichEdit;
    PopupMenu: TPopupMenu;
    ItemSystemFont: TMenuItem;
    ItemCourierFont: TMenuItem;
    ItemPrint: TMenuItem;
    ItemClose: TMenuItem;
    N2: TMenuItem;
    N1: TMenuItem;
    ItemSavetoFile: TMenuItem;
    procedure MemoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ItemPrintClick(Sender: TObject);
    procedure ItemSystemFontClick(Sender: TObject);
    procedure ItemCourierFontClick(Sender: TObject);
    procedure ItemCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ItemSavetoFileClick(Sender: TObject);
  private
  	FListIndex: Integer;
    FTickEnabled: Boolean;
    class procedure NoMatch(			Sender: TObject;
  													const Info: TObject);
  protected
 		class function Match(const Info: TObject): Boolean; virtual;
    procedure WMSysCommand(var Message: TMessage); message WM_SYSCOMMAND;
    procedure HandleTick; virtual; abstract;
    procedure SaveImage(TicketImage: TStringList); virtual;
  public
  	property ListIndex: Integer read FListIndex write FListIndex;
    property TickEnabled: Boolean read FTickEnabled write FTickEnabled;
 		class	procedure Register;
 	  class function DerivedClass(const Info: TFormTicketCreateInfo): TFormTicketClass;
    procedure Tick;
    procedure Save(       IniFile: TIniFile;
    							 const	Section: string); virtual;
    procedure Restore(      IniFile: TIniFile;
    							 		const	Section: string); virtual;
    procedure Assign(Source: TPersistent); override;
    procedure SaveToFile;
    procedure Print;
  end;
  EFormTicket = class(Exception);
  EUnsupportedTicket = class(EFormTicket);

implementation

{$R *.DFM}

uses
	Main, Graphics;

const
	EUnsupportedTicketMsg = ' - Unsupported ticket type';
  ProportionalFontName = 'System';
  FixedFontName = 'Courier';
  ProportionalFontStyle = [fsBold];
  FixedFontStyle = [];
  FontSize = 10;

var
	ClassDeriver: TDeriver;

class procedure TFormTicket.NoMatch(		 	Sender: TObject;
							  										const Info: TObject);
begin

	with Info as TFormTicketCreateInfo do
		raise EUnsupportedTicket.Create(Self.ClassName + EUnsupportedTicketMsg);

end;

class function TFormTicket.Match(const Info: TObject): Boolean;
begin

	with Info as TFormTicketCreateInfo do
  	Result := (CompareText(IniFile.ReadString(Section,'Type',''),Self.ClassName) = 0)

end;

class	procedure TFormTicket.Register;
begin

	if ClassDeriver = nil then begin
		ClassDeriver := TDeriver.Create;
  	ClassDeriver.OnNoMatch := TFormTicket.NoMatch;
  end;
	ClassDeriver.Register(Self,Self.Match);

end;

class function TFormTicket.DerivedClass(const Info: TFormTicketCreateInfo): TFormTicketClass;
begin

	Result := TFormTicketClass(ClassDeriver.DerivedClass(Info));

end;

procedure TFormTicket.WMSysCommand(var Message: TMessage);
begin

	if Message.WParam = SC_MINIMIZE then begin
  	Tag := -1;
  	Hide
  end
  else if Message.WParam = SC_CLOSE then begin
    if Memo.Lines.Count = 0 then
	  	inherited
    else
    	PopupMenu.Popup(Message.LParamLo,Message.LParamHi);
  end
  else
  	inherited;

end;

procedure TFormTicket.Tick;
begin

  if FTickEnabled then
    HandleTick;

end;

procedure TFormTicket.Save(				IniFile: TIniFile;
    							 				 const 	Section: string);

var
  S: string;
	I: Integer;

begin

	with IniFile do begin
    EraseSection(Section);
  	WriteString(Section,'Type',Self.ClassName);
  	WriteString(Section,'Caption',Self.Caption);
  	WriteString(Section,'Font',Memo.Font.Name);
		WriteInteger(Section,'Top',Top);
		WriteInteger(Section,'Left',Left);
		WriteInteger(Section,'Width',Width);
	  WriteInteger(Section,'Height',Height);
	  WriteBool(Section,'Minimized',Tag < 0);
    WriteInteger(Section,'N',Memo.Lines.Count);
    for I := 0 to Memo.Lines.Count - 1 do begin
			S := Memo.Lines[I];
      if Length(S) = 0 then
      	WriteString(Section,Format('%2.2d',[I]),'')
      else if S[1] = ' ' then
      	WriteString(Section,Format('%2.2d',[I]),'@' + S)
      else if S[1] = '@' then
      	WriteString(Section,Format('%2.2d',[I]),'@' + S)
      else
			 	WriteString(Section,Format('%2.2d',[I]),S);
    end;
  end;

end;

procedure TFormTicket.Restore(			IniFile: TIniFile;
 							 								const	Section: string);

var
  S: string;
	I: Integer;

begin

	with IniFile do begin
	  Caption := ReadString(Section,'Caption','');
    Width := ReadInteger(Section,'Width',0);
  	Height := ReadInteger(Section,'Height',0);
   	if ReadBool(Section,'Minimized',false) then begin
		  PostMessage(Handle,WM_SYSCOMMAND,SC_MINIMIZE,0);
      Tag := -1;
    end
   	else
      Tag := 0;
   	Top := ReadInteger(Section,'Top',0);
    Left := ReadInteger(Section,'Left',0);
    for I := 0 to ReadInteger(Section,'N',0) - 1 do begin
      S := ReadString(Section,Format('%2.2d',[I]),'');
      if Length(S) = 0 then
    		Memo.Lines.Add('')
      else if S[1] = '@' then
    		Memo.Lines.Add(Copy(S,2,Length(S) - 1))
      else
    		Memo.Lines.Add(S);
    end;
		with Memo.Font do begin
    	Name := ReadString(Section,'Font',ProportionalFontName);
    	Size := FontSize;
      ItemSystemFont.Visible := not (Name = ProportionalFontName);
		  ItemCourierFont.Visible := not (Name = FixedFontName);
      if Name = ProportionalFontName then
      	Style := ProportionalFontStyle
      else
      	Style := FixedFontStyle;
    end;
	end;

end;

procedure TFormTicket.Assign(Source: TPersistent);
begin

	with Source as TFormTicket do begin
  	Self.Memo.Lines.Assign(Memo.Lines);
  	Self.OnClose := OnClose;
   	Self.OnPaint := OnPaint;
    Self.OnDeactivate := OnDeactivate;
  end;

end;

procedure TFormTicket.SaveImage(TicketImage: TStringList);
begin

	with TicketImage do begin
    Assign(Memo.Lines);
    Insert(0,'Caption: ' + Caption);
    Insert(1,'');
  end;

end;

procedure TFormTicket.SaveToFile;

var
  TicketImage: TStringList;

begin

  TicketImage := TStringList.Create;
  SaveImage(TicketImage);
  FormMain.SaveToFile(TicketImage,Caption);
  TicketImage.Free;

end;

procedure TFormTicket.Print;

var
  TicketImage: TStringList;

begin

  TicketImage := TStringList.Create;
  SaveImage(TicketImage);
  FormMain.Print(TicketImage,Memo.Font);
  TicketImage.Free;

end;

procedure TFormTicket.MemoMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin

	Tag := 0;

end;

procedure TFormTicket.ItemSystemFontClick(Sender: TObject);
begin

	with Memo.Font do begin
		Name := ProportionalFontName;
  	Size := FontSize;
    Style := ProportionalFontStyle;
  end;
  ItemSystemFont.Visible := false;
  ItemCourierFont.Visible := true;

end;

procedure TFormTicket.ItemCourierFontClick(Sender: TObject);
begin

	with Memo.Font do begin
    Style := FixedFontStyle;
  	Size := FontSize;
		Name := FixedFontName;
	end;
  ItemSystemFont.Visible := true;
  ItemCourierFont.Visible := false;

end;

procedure TFormTicket.ItemSavetoFileClick(Sender: TObject);
begin

  SaveToFile;

end;

procedure TFormTicket.ItemPrintClick(Sender: TObject);
begin

  ItemPrint.Enabled := false;
	Print;
  ItemPrint.Enabled := true;

end;

procedure TFormTicket.ItemCloseClick(Sender: TObject);
begin

	Close;

end;

procedure TFormTicket.FormShow(Sender: TObject);
begin

	Tag := 0;

end;

procedure TFormTicket.FormResize(Sender: TObject);
begin

	Memo.Refresh;

end;

procedure TFormTicket.FormCreate(Sender: TObject);
begin

  FTickEnabled := false;
  Tag := 0;
	with Memo.Font do begin
   	Name := ProportionalFontName;
   	Size := FontSize;
    ItemSystemFont.Visible := false;
    ItemCourierFont.Visible := true;
   	Style := ProportionalFontStyle
  end;

end;

initialization

	TFormTicket.Register;

end.

